#include "appcgroup.h"
#include <QDBusPendingReply>
#include <QDebug>
#include <unistd.h>

AppCGroup::AppCGroup(QObject *parent) 
    : QObject(parent)
    , m_processManagerInterface(new com::kylin::ProcessManager("com.kylin.ProcessManager", 
                                "/com/kylin/ProcessManager", QDBusConnection::systemBus()))
{

}

QString AppCGroup::createProcessCGroup(const QString &appId, const QList<int> &pids)
{
    auto reply = m_processManagerInterface->CreateProcessCGroup(appId, pids);
    qDebug() << "reply is finished. " << reply.isFinished();
    while (!reply.isFinished()) {
        qApp->processEvents();
        usleep(10);
    }
    if (reply.isError()) {
        qWarning() << "createProcessCGroup error, " << reply.error();
        return QString();
    }

    if (reply.value().value(kDbusResult).toString().isEmpty()) {
        qWarning() << "createProcessCGroup error, " << reply.value().value(kDbusErrMsg).toString();
        return QString();
    }
    return reply.value().value(kDbusResult).toString();
}

bool AppCGroup::deleteProcessCGroup(const QString &cgroupName)
{
    auto reply = m_processManagerInterface->DeleteProcessCGroup(cgroupName);
    return true;   
}

bool AppCGroup::setProcessCGroupResourceLimit(const QString &cgroupName,
                                              const QString &attrName,
                                              int value)
{
    auto reply = m_processManagerInterface->SetProcessCGroupResourceLimit(cgroupName, attrName, value);
    return true; 
}

bool AppCGroup::reclaimProcesses(const QList<int> &pids)
{
    auto reply = m_processManagerInterface->ReclaimProcesses(pids);
    return true;   
}

bool AppCGroup::reclaimProcesses(const QString &cgroupName)
{
    auto reply = m_processManagerInterface->ReclaimProcesses(cgroupName);
    return true; 
}

QString AppCGroup::cgroupNameWithPid(int pid)
{
    auto reply = m_processManagerInterface->CGroupNameWithPid(pid);
    while (!reply.isFinished()) {
        qApp->processEvents();
        sleep(1);
    }
    if (reply.isError()) {
        qWarning() << __FUNCTION__ << "error, " << reply.error();
        return QString();
    }
    return reply.value().value(kDbusResult).toString();
}
