#ifndef APPSTATUSMANAGER_H
#define APPSTATUSMANAGER_H

#include <QObject>
#include <QMap>
#include "timerwheel.h"
#include "appinfomanager.h"
#include "appcgroup.h"
#include "dbusservices/whitelistmanager.h"
#include "common.h"

class Policy;
class AppInfoManager;
class AppStatusManager : public QObject
{
    Q_OBJECT
public:
    explicit AppStatusManager(AppInfoManager *parent);

    /**
     * @brief newAppInstance
     * @param appId
     * @param forkPid
     * @param instlndex
     * @return
     */
    QString newAppInstance(const QString &appId, const QList<int> &pids);

    /**
     * @brief deleteAppCGroup
     * @param cgroupName
     * @return
     */
    bool deleteAppCGroup(const QString &cgroupName);

    void appFinished(const QString &appId, const QString &instName);

    /**
     * @brief activeAppChanged
     * @param currentAppId
     * @param preAppId
     */
    void activeAppChanged(const QString &currentAppId, 
                          const QString &currentInstName,
                          const QString &preAppId, 
                          const QString &preInstName,
                          bool preAppMinimized);

    void appWidsIsEmpty(const QString &appId, const QString &instance);

    void appStatusTimeOutCallback(timerwheel::AppData appData);

    void resourceThresholdWarning(Policy::Feature resource, Policy::ResourceUrgency level);
    void tabletModeChanged(bool isTablet);

    QString cgroupWithPid(int pid);

    bool thawApp(const QString &appId, const QString &cgroupName);

    void aboutToQuitApp();

    void initConnections();

private Q_SLOTS:
    void updateWhitelistAppStatus(QString desktopName);

private:
    void callMprisDbus(const QString &dbusService);
    bool setProcessResourceValue(const QString &desktopFile,
                                 const QString &cgroupName,
                                 const QString &attrName,
                                 int value);

private:
    timerwheel::TimerWheel *m_timerWheel;
    QMap<QString, QMap<QString, int>> m_cgroupInfo;
    AppInfoManager *m_appInfoManager;
    AppCGroup *m_appCGroup;
    Policy *m_policy;
    WhiteListManager &m_whiteListManager = common::Singleton<WhiteListManager>::GetInstance();
};

#endif // APPSTATUSMANAGER_H
