#ifndef TIMERWHEEL_H
#define TIMERWHEEL_H

#include <functional>
#include "policy.h"

class QTimer;
namespace timerwheel {

struct AppData
{
    QString appId;
    QString instName;
    Policy::AppStatus dstStatus;
};

struct Timer
{
    quint64 timerId;
    int rotation;
    int tickSlot;
    std::function<void(AppData)> callback;
    AppData appData;
};

class TimerWheel : public QObject
{
public:
    explicit TimerWheel(QObject *parent = nullptr);
    quint64 addTimer(int timeout,
                     AppData appData,
                     const std::function<void(AppData)> &callback);

    bool deleteTimer(quint64 timerId);

private Q_SLOTS:
    void tick();

private:
    QTimer *m_timer;
    int m_currentSlot;
    quint64 m_currentTimerCount;
    QMap<int, QList<Timer *>> m_slotTimers;
    const int kSi = 1; // 1s
    const int kSlotNumber = 60;
};

} // namsapce timerwheol;


#endif // TIMERWHEEL_H
