#include "mainwindow.h"
#include "ui_mainwindow.h"
#include <QDesktopWidget>
#include <QTime>
#include "kylinvirtualkeyboardinterface.h"

#define THEME_QT_SCHEMA "org.ukui.style"
#define THEME_Style_Name "styleName"
#include <QString>
#include <QProcess>
#include <QStringList>

MainWindow::MainWindow(QWidget *parent)
    : QMainWindow(parent)
    , ui(new Ui::MainWindow)
{

    ui->setupUi(this);
    setWindowFlags(Qt::Tool |Qt::WindowStaysOnTopHint |Qt::FramelessWindowHint |Qt::X11BypassWindowManagerHint);
    setAttribute(Qt::WA_TranslucentBackground);

    QDesktopWidget* desktopWidget = QApplication::desktop();
    QRect clientRect = desktopWidget->availableGeometry();
    this->move(clientRect.width()-this->width(),clientRect.height()*0.5);
    this->setStyleSheet(" background-color:rgba(255,255,255,255); border-radius: 28px;");
    ui->pushButton->setStyleSheet("background-color:rgba(255,255,255,255); border-radius: 28px;");

    p_ftTabletFlag = ftTabletFlag(); //获取飞腾平版设备型号  初始化的时候获取一次即可

    registerDbusInterface();    //连接状态管理器dbus信号
    keyboardStatusChange();//
    registerGsettingsInterface();

    ui->pushButton->installEventFilter(this);

}

MainWindow::~MainWindow()
{
    delete ui;
    delete p_pushButton;
    delete sougouInterface;
    delete statusManagerInterface;
    delete kylinVKinterface;
}

void MainWindow::onPushButtonClicked()
{
    if (sougouInterface->p_sougouInterface->isValid()) {
        if(!sougouInterface->isKeyboardVisible()) {
            sougouInterface->switchInputToTouchMode();
        }
    } else if(kylinVKinterface->kylinVirtualKeyboardInterface->isValid()) {
        //使用自己本身的虚拟键盘
        kylinVKinterface->toShowVirtualKeyboard();
    } else {
        qDebug()<<"hoverbutton onPushButtonClicked is error...";
    }
}

void MainWindow::registerDbusInterface()
{
    sougouInterface = new SougouInterface();
    statusManagerInterface = new StatusManagerInterface();
    kylinVKinterface = new KylinVirtualKeyboard();

    QDBusConnection::sessionBus().connect("com.kylin.statusmanager.interface",
                                          "/",
                                          "com.kylin.statusmanager.interface",
                                          "inputmethod_change_signal",
                                          this,
                                          SLOT(keyboardStatusChange()));
}

void MainWindow::registerGsettingsInterface()
{
    QByteArray styleid(THEME_QT_SCHEMA);
    if(QGSettings::isSchemaInstalled(styleid))
    {
        p_styleGsetting = new QGSettings(styleid);
        // 监听当前主题变化
        connect(p_styleGsetting, &QGSettings::changed, [=](const QString & key) {
            if (key == THEME_Style_Name) {
                if(p_styleGsetting->get(THEME_Style_Name).toString()=="ukui-light") {
                    ui->pushButton->setIcon(QIcon(":/keyboard_light.svg"));
                }else if (p_styleGsetting->get(THEME_Style_Name).toString()=="ukui-dark") {
                    ui->pushButton->setIcon(QIcon(":/keyboard_dark.svg"));
                }
            }
        });
        ui->pushButton->setIcon(QIcon(":/keyboard_light.svg"));

    }else {
        qDebug()<<" style gsetting is not install";
    }
}

void MainWindow::keyboardStatusChange()
{
    int reply = statusManagerInterface->getInputmethodMode();
    if (reply == 1 && !p_ftTabletFlag) {
        this->hide();
        return;
    }
    this->show();
}

bool MainWindow::eventFilter(QObject *watched, QEvent *event)
{
    static QPoint lastPoint;
    static bool isPressed = false;
    if(watched == ui->pushButton) {
        if (event->type() == QEvent::MouseButtonPress) {
            QMouseEvent *e = static_cast<QMouseEvent *>(event);
            if (this->rect().contains(e->pos()) && (e->button() == Qt::LeftButton)) {
                lastPoint = e->pos();
                isPressed = true;
            }
        } else if (event->type() == QEvent::MouseMove) {
            QMouseEvent *e = static_cast<QMouseEvent *>(event);
            int dx = e->pos().x() - lastPoint.x();
            int dy = e->pos().y() - lastPoint.y();

            this->move(this->x() + dx, this->y() + dy);
            isPressed = false;
            return true;

        } else if (event->type() == QEvent::MouseButtonRelease && isPressed) {
            onPushButtonClicked();
            isPressed = false;
        }
    }
    return QWidget::eventFilter(watched, event);
}
bool MainWindow::ftTabletFlag()
{
    if (getSysVendor() == "PANDING LTD" && getProductFamily() == "tablet" ) {
        qWarning()<<" ft tablet";
       return true;
    } else {
        return false;
    }

}

QString MainWindow::getSysVendor()
{
    QProcess process;
    QStringList options;
    options << "-c";
    options << "cat /sys/class/dmi/id/sys_vendor";
    process.start("/bin/bash", options);
    process.waitForFinished();
    QString result = process.readAllStandardOutput();

    QStringList list = result.split("\n");
    result=list.at(0);
    return result;
}
QString MainWindow::getProductFamily()
{
    QProcess process;
    QStringList options;
    options << "-c";
    options << "cat /sys/class/dmi/id/product_family";
    process.start("/bin/bash", options);
    process.waitForFinished();
    QString result = process.readAllStandardOutput();

    QStringList list = result.split("\n");
    result=list.at(0);
    return result;
}
